@@ -1,54 +1,69 @@
 use std::array::IntoIter;
+use std::path::Path;
+use std::io::Write;
+use std::fs::OpenOptions;
 use std::collections::HashMap;
 use std::iter::FromIterator;
+use std::process::Command;
 use crate::error::RadError;
-use crate::consts::{MAIN_CALLER, LINE_ENDING};
+use crate::consts::{MAIN_CALLER, TEMP_PATH, TEMP_FILE};
 use regex::Regex;
 use crate::utils::Utils;
 use crate::processor::Processor;
+use crate::formatter::Formatter;
 use lipsum::lipsum;
+use lazy_static::lazy_static;
 
 type MacroType = fn(&str, &mut Processor) -> Result<String, RadError>;
 
-#[derive(Clone)]
-pub struct BasicMacro<'a> {
-    macros : HashMap<&'a str, MacroType>,
+lazy_static!{
+   pub static ref ITER: Regex = Regex::new(r"\$:").unwrap();
 }
 
-impl<'a> BasicMacro<'a> {
+#[derive(Clone)]
+pub struct BasicMacro {
+    macros : HashMap<String, MacroType>,
+}
+
+impl BasicMacro {
     pub fn new() -> Self {
         // Create hashmap of functions
         let map = HashMap::from_iter(IntoIter::new([
-            ("rsub", BasicMacro::regex_sub as MacroType),
-            ("rdel", BasicMacro::regex_del as MacroType),
-            ("eval", BasicMacro::eval as MacroType),
-            ("trim", BasicMacro::trim as MacroType),
-            ("chomp", BasicMacro::chomp as MacroType),
-            ("comp", BasicMacro::compress as MacroType),
-            ("lipsum", BasicMacro::placeholder as MacroType),
-            ("time", BasicMacro::time as MacroType),
-            ("date", BasicMacro::date as MacroType),
-            ("include", BasicMacro::include as MacroType),
-            ("repeat", BasicMacro::repeat as MacroType),
-            ("syscmd", BasicMacro::syscmd as MacroType),
-            ("ifelse", BasicMacro::ifelse as MacroType),
-            ("ifdef", BasicMacro::ifdef as MacroType),
-            ("foreach", BasicMacro::foreach as MacroType),
-            ("forloop", BasicMacro::forloop as MacroType),
-            ("undef", BasicMacro::undef as MacroType),
+            ("regex".to_owned(), BasicMacro::regex_sub as MacroType),
+            ("eval".to_owned(), BasicMacro::eval as MacroType),
+            ("trim".to_owned(), BasicMacro::trim as MacroType),
+            ("chomp".to_owned(), BasicMacro::chomp as MacroType).to_owned(),
+            ("comp".to_owned(), BasicMacro::compress as MacroType).to_owned(),
+            ("lipsum".to_owned(), BasicMacro::placeholder as MacroType).to_owned(),
+            ("time".to_owned(), BasicMacro::time as MacroType).to_owned(),
+            ("date".to_owned(), BasicMacro::date as MacroType).to_owned(),
+            ("include".to_owned(), BasicMacro::include as MacroType).to_owned(),
+            ("repeat".to_owned(), BasicMacro::repeat as MacroType).to_owned(),
+            ("syscmd".to_owned(), BasicMacro::syscmd as MacroType).to_owned(),
+            ("ifelse".to_owned(), BasicMacro::ifelse as MacroType).to_owned(),
+            ("ifdef".to_owned(), BasicMacro::ifdef as MacroType).to_owned(),
+            ("foreach".to_owned(), BasicMacro::foreach as MacroType).to_owned(),
+            ("forloop".to_owned(), BasicMacro::forloop as MacroType).to_owned(),
+            ("undef".to_owned(), BasicMacro::undefine_call as MacroType).to_owned(),
+            ("rename".to_owned(), BasicMacro::rename_call as MacroType).to_owned(),
+            ("append".to_owned(), BasicMacro::append as MacroType).to_owned(),
+            ("from".to_owned(), BasicMacro::from_data as MacroType).to_owned(),
+            ("table".to_owned(), BasicMacro::table as MacroType).to_owned(),
+            ("len".to_owned(), BasicMacro::len as MacroType).to_owned(),
+            ("tr".to_owned(), BasicMacro::translate as MacroType).to_owned(),
+            ("sub".to_owned(), BasicMacro::substring as MacroType).to_owned(),
+            ("pause".to_owned(), BasicMacro::pause as MacroType).to_owned(),
+            ("temp".to_owned(), BasicMacro::temp as MacroType).to_owned(),
+            ("-".to_owned(), BasicMacro::get_pipe as MacroType).to_owned(),
         ]));
         // Return struct
-        Self {  macros : map}
+        Self { macros : map }
     }
 
     pub fn contains(&self, name: &str) -> bool {
         self.macros.contains_key(name)
     }
 
-    pub fn unset(&mut self, name: &str) {
-        self.macros.remove(name);
-    }
-
     pub fn call(&self, name : &str, args: &str, processor: &mut Processor) -> Result<String, RadError> {
         if let Some(func) = self.macros.get(name) {
             // Print out macro call result
@@ -59,6 +74,18 @@ impl<'a> BasicMacro<'a> {
         }
     }
 
+    pub fn undefine(&mut self, name: &str) {
+        self.macros.remove(name);
+    }
+
+    pub fn rename(&mut self, name: &str, target: &str) {
+        let func = self.macros.remove(name).unwrap();
+        self.macros.insert(target.to_owned(), func);
+    }
+
+    // ==========
+    // Basic Macros
+    // ==========
     fn time(_: &str, _ : &mut Processor) -> Result<String, RadError> {
         Ok(format!("{}", chrono::offset::Local::now().format("%H:%M:%S")))
     }
@@ -88,26 +115,6 @@ impl<'a> BasicMacro<'a> {
         }
     }
 
-    fn regex_del(args: &str, processor: &mut Processor) -> Result<String, RadError> {
-        let args = &processor.parse_chunk(
-            1000, 
-            &MAIN_CALLER.to_owned(), 
-            args
-        )?;
-
-        if let Some(args) = Utils::args_with_len(args, 2) {
-            let source = &args[0];
-            let target = &args[1];
-
-            // This is regex expression without any preceding and trailing commands
-            let reg = Regex::new(&format!(r"{}", target))?;
-            let result = reg.replace_all(source, ""); // This is a cow, moo~, btw this replaces all match as empty character which technically deletes matches
-            Ok(result.to_string())
-        } else {
-            Err(RadError::InvalidArgument("Regex del requires two arguments"))
-        }
-    }
-
     fn eval(args: &str, processor: &mut Processor ) -> Result<String, RadError> {
         let args = &processor.parse_chunk(
             1000, 
@@ -118,9 +125,11 @@ impl<'a> BasicMacro<'a> {
         if let Some(args) = Utils::args_with_len(args, 1) {
             let formula = &args[0];
             let result = evalexpr::eval(formula)?;
+            // TODO
+            // Enable floating points length (or something similar)
             Ok(result.to_string())
         } else {
-            Err(RadError::InvalidArgument("Regex del requires an argument"))
+            Err(RadError::InvalidArgument("Eval requires an argument"))
         }
     }
 
@@ -145,8 +154,8 @@ impl<'a> BasicMacro<'a> {
 
         if let Some(args) = Utils::args_with_len(args, 1) {
             let source = &args[0];
-            let reg = Regex::new(&format!(r"{0}\s*{0}", LINE_ENDING))?;
-            let result = reg.replace_all(source, &format!("{0}{0}", LINE_ENDING));
+            let reg = Regex::new(&format!(r"{0}\s*{0}", &processor.newline))?;
+            let result = reg.replace_all(source, &format!("{0}{0}", &processor.newline));
 
             Ok(result.to_string())
         } else {
@@ -232,27 +241,28 @@ impl<'a> BasicMacro<'a> {
     }
 
     // $syscmd(echo 'this is printed')
-    fn syscmd(args: &str, processor: &mut Processor) -> Result<String, RadError> {
-        let args = &processor.parse_chunk(
-            1000, 
-            &MAIN_CALLER.to_owned(), 
-            args
-        )?;
-
+    fn syscmd(args: &str, _: &mut Processor) -> Result<String, RadError> {
         if let Some(args_content) = Utils::args_with_len(args, 1) {
             let source = &args_content[0];
             let arg_vec = Utils::args_to_vec(&source, ' ', ('\'', '\''));
 
-            let cmd = &arg_vec[0];
-            let cmd_args = if arg_vec.len() >= 2 {&arg_vec[1..]} else {&[]};
+            let output = if cfg!(target_os = "windows") {
+                Command::new("cmd")
+                    .arg("/C")
+                    .args(arg_vec)
+                    .output()
+                    .expect("failed to execute process")
+                    .stdout
+            } else {
+                let sys_args = if arg_vec.len() > 1 { &arg_vec[1..] } else { &[] };
+                Command::new(&arg_vec[0])
+                    .args(sys_args)
+                    .output()
+                    .expect("failed to execute process")
+                    .stdout
+            };
 
-            let output = String::from_utf8(
-                std::process::Command::new(cmd)
-                .args(cmd_args)
-                .output()
-                .expect(&format!("Failed to execute command {}", args)).stdout
-            )?;
-            Ok(output)
+            Ok(String::from_utf8(output)?)
         } else {
             Err(RadError::InvalidArgument("Syscmd requires an argument"))
         }
@@ -277,7 +287,7 @@ impl<'a> BasicMacro<'a> {
             } else if let Ok(number) = trimmed_cond.parse::<i32>() {
                 if number != 0 { return Ok(if_state.to_owned()); }
             } else {
-                return Err(RadError::InvalidArgument("Ifelse requires true/fals or zero/nonzero."))
+                return Err(RadError::InvalidArgument("Ifelse requires either true/false or zero/nonzero integer."))
             }
             // if else statement exsits
             if args.len() >= 3 {
@@ -291,7 +301,7 @@ impl<'a> BasicMacro<'a> {
 
             Ok(String::new())
         } else {
-            Err(RadError::InvalidArgument("Syscmd requires an argument"))
+            Err(RadError::InvalidArgument("ifelse requires an argument"))
         }
     }
 
@@ -316,11 +326,11 @@ impl<'a> BasicMacro<'a> {
                 Ok("false".to_owned())
             }
         } else {
-            Err(RadError::InvalidArgument("Syscmd requires an argument"))
+            Err(RadError::InvalidArgument("Ifdef requires an argument"))
         }
     }
 
-    fn undef(args: &str, processor: &mut Processor) -> Result<String, RadError> {
+    fn undefine_call(args: &str, processor: &mut Processor) -> Result<String, RadError> {
         let args = &processor.parse_chunk(
             1000, 
             &MAIN_CALLER.to_owned(), 
@@ -330,21 +340,15 @@ impl<'a> BasicMacro<'a> {
         if let Some(args) = Utils::args_with_len(args, 1) {
             let name = &args[0];
 
-            // Return true or false by the definition
-            if processor.map.basic.contains(name) {
-                processor.map.basic.unset(name);
-            }
-            if processor.map.custom.contains_key(name) {
-                processor.map.custom.remove(name);
-            }
+            processor.map.undefine(name);
             Ok("".to_owned())
         } else {
-            Err(RadError::InvalidArgument("Syscmd requires an argument"))
+            Err(RadError::InvalidArgument("Undefine requires an argument"))
         }
     }
 
     // $foreach()
-    // $foreach($testo($_()),"a,b,c")
+    // $foreach("a,b,c",$:)
     fn foreach(args: &str, processor: &mut Processor) -> Result<String, RadError> {
         if let Some(args) = Utils::args_with_len(args, 2) {
             let mut sums = String::new();
@@ -358,7 +362,7 @@ impl<'a> BasicMacro<'a> {
             let processed = processor.parse_chunk(0, &MAIN_CALLER.to_owned(),&target)?;
 
             for value in loopable.split(',') {
-                sums.push_str(&processed.replace("$_", value));
+                sums.push_str(&ITER.replace_all(&processed, value));
             }
             Ok(sums)
         } else {
@@ -366,7 +370,7 @@ impl<'a> BasicMacro<'a> {
         }
     }
 
-    // $forloop("1,5",$testo($_))
+    // $forloop("1,5",$:)
     fn forloop(args: &str, processor: &mut Processor) -> Result<String, RadError> {
         if let Some(args) = Utils::args_with_len(args, 2) {
             let mut sums = String::new();
@@ -380,7 +384,7 @@ impl<'a> BasicMacro<'a> {
             let loopable = loopable.split(',').collect::<Vec<&str>>();
 
             if loopable.len() != 2 {
-                RadError::InvalidArgument("Forloop's second argument should be quoted min,max value e.g \"2,5\"");
+                RadError::InvalidArgument("Forloop's first argument should be quoted min,max value e.g \"2,5\"");
             }
             let min: usize; 
             let max: usize; 
@@ -394,31 +398,181 @@ impl<'a> BasicMacro<'a> {
             let processed = processor.parse_chunk(0, &MAIN_CALLER.to_owned(), &target)?;
 
             for value in min..=max {
-                sums.push_str(&processed.replace("$_", &value.to_string()));
+                sums.push_str(&ITER.replace_all(&processed, &value.to_string()));
             }
 
             Ok(sums)
         } else {
-            Err(RadError::InvalidArgument("Foreach requires two argument"))
+            Err(RadError::InvalidArgument("Forloop requires two argument"))
         }
     }
 
-    // $from($_,"1,2,34,5,6")
-    #[allow(dead_code)]
-    fn from_data() {
+    // $from("1,2,3\n4,5,6", macro_name)
+    fn from_data(args: &str, processor: &mut Processor) -> Result<String, RadError> {
+        if let Some(args) = Utils::args_with_len(args, 2) {
+            let macro_data = &args[0];
+            let macro_name = &Utils::trim(&args[1])?;
 
+            let result = Formatter::csv_to_macros(macro_name, macro_data, &processor.newline)?;
+            let result = processor.parse_chunk(0, &MAIN_CALLER.to_owned(), &result)?;
+            Ok(result)
+        } else {
+            Err(RadError::InvalidArgument("From requires two arguments"))
+        }
     }
 
-    // TODO
-    #[allow(dead_code)]
-    fn csv(args: &str, _processor: &mut Processor) -> Result<String, RadError> {
-        if let Some(args) = Utils::args_with_len(args, 3) {
-            let _table_format = &args[0]; // Either gfm, wikitex, latex, none
-            let _csv_query = &args[1];
-            let _csv_content = &args[2];
+    fn table(args: &str, p: &mut Processor) -> Result<String, RadError> {
+        if let Some(args) = Utils::args_with_len(args, 2) {
+            let table_format = &args[0]; // Either gfm, wikitex, latex, none
+            let csv_content = &args[1];
+            let result = Formatter::csv_to_table(table_format, csv_content, &p.newline)?;
+            Ok(result)
+        } else {
+            Err(RadError::InvalidArgument("Table requires two arguments"))
+        }
+    }
+
+    fn get_pipe(_: &str, processor: &mut Processor) -> Result<String, RadError> {
+        let out = processor.pipe_value.clone();
+        processor.pipe_value.clear();
+        Ok(out)
+    }
+
+    /// Return a length of the string
+    /// This is O(n) operation
+    /// String.len() function returns byte length not "Character" length
+    /// therefore, chars().count() is used
+    fn len(args: &str, _: &mut Processor) -> Result<String, RadError> {
+        Ok(args.chars().count().to_string())
+    }
+
+    fn rename_call(args: &str, processor: &mut Processor) -> Result<String, RadError> {
+        if let Some(args) = Utils::args_with_len(args, 2) {
+            let target = &args[0];
+            let new = &args[1];
+            processor.map.rename(target, new);
+
             Ok(String::new())
         } else {
-            Err(RadError::InvalidArgument("Syscmd requires an argument"))
+            Err(RadError::InvalidArgument("Rename requires two arguments"))
+        }
+    }
+
+    fn append(args: &str, processor: &mut Processor) -> Result<String, RadError> {
+        if let Some(args) = Utils::args_with_len(args, 2) {
+            let name = &args[0];
+            let target = &args[1];
+            processor.map.append(name, target);
+
+            Ok(String::new())
+        } else {
+            Err(RadError::InvalidArgument("Append requires two arguments"))
+        }
+    }
+    fn translate(args: &str, _: &mut Processor) -> Result<String, RadError> {
+        if let Some(args) = Utils::args_with_len(args, 3) {
+            let mut source = args[0].clone();
+            let target = &args[1].chars().collect::<Vec<char>>();
+            let destination = &args[2].chars().collect::<Vec<char>>();
+
+            if target.len() != destination.len() {
+                return Err(RadError::InvalidArgument("Tr's replacment should have same length of texts"));
+            }
+
+            for i in 0..target.len() {
+                source = source.replace(target[i], &destination[i].to_string());
+            }
+
+            Ok(source)
+        } else {
+            Err(RadError::InvalidArgument("Tr requires two arguments"))
+        }
+    }
+
+    // $sub(",",GivenString)
+    fn substring(args: &str, _: &mut Processor) -> Result<String, RadError> {
+        if let Some(args) = Utils::args_with_len(args, 2) {
+            let source = &args[1];
+            let loopable = &args[0].split(',').collect::<Vec<&str>>();
+
+            if loopable.len() != 2 {
+                RadError::InvalidArgument("Substring's first argument should be quoted min,max value e.g \"2,5\"");
+            }
+
+            let mut min: Option<usize> = None;
+            let mut max: Option<usize> = None;
+
+            let start = Utils::trim(loopable[0])?;
+            let end = Utils::trim(loopable[1])?;
+            if let Ok(num) = start.parse::<usize>() {
+                min.replace(num);
+            } else { 
+                if start.len() != 0 {
+                    return Err(RadError::InvalidArgument("Sub's min value should be non zero positive integer or empty value")); 
+                }
+            }
+            if let Ok(num) = end.parse::<usize>() {
+                max.replace(num);
+            } else { 
+                if end.len() != 0 {
+                    return Err(RadError::InvalidArgument("Sub's max value should be non zero positive integer or empty value")); 
+                }
+            }
+            if let Some(min) = min {
+                if let Some(max) = max { // Both
+                    Ok(source[min..max].to_string())
+                } else { // no max
+                    Ok(source[min..].to_string())
+                }
+            } else { // No min
+                if let Some(max) = max { // at least max 
+                    Ok(source[..max].to_string())
+                } else { // Nothing 
+                    Ok(source[..].to_string())
+                }
+            }
+        } else {
+            Err(RadError::InvalidArgument("Sub requires some arguments"))
+        }
+    }
+    fn pause(args: &str, processor : &mut Processor) -> Result<String, RadError> {
+        if let Some(args) = Utils::args_with_len(args, 1) {
+            let arg = &args[0];
+            if let Ok(value) =Utils::is_arg_true(arg) {
+                if value {
+                    processor.paused = true;
+                } else {
+                    processor.paused = false;
+                }
+                Ok(String::new())
+            } 
+            // Failed to evaluate
+            else {
+                Err(RadError::InvalidArgument("Pause requires either true/false or zero/nonzero integer."))
+            }
+        } else {
+            Err(RadError::InvalidArgument("Pause requires an argument"))
+        }
+    }
+    fn temp(args: &str, _: &mut Processor) -> Result<String, RadError> {
+        if let Some(args) = Utils::args_with_len(args, 1) {
+            let truncate = &args[0];
+            let content = &args[1];
+            if let Ok(value) = Utils::is_arg_true(truncate) {
+                let file = Path::new(TEMP_PATH).join(TEMP_FILE);
+                let mut temp_file = OpenOptions::new()
+                    .create(true)
+                    .write(true)
+                    .truncate(value)
+                    .open(file)
+                    .unwrap();
+                temp_file.write_all(content.as_bytes())?;
+                Ok(String::new())
+            } else {
+                Err(RadError::InvalidArgument("Temp requires either true/false or zero/nonzero integer."))
+            }
+        } else {
+            Err(RadError::InvalidArgument("Temp requires an argument"))
         }
     }
 }
