@@ -1,6 +1,7 @@
 use clap::clap_app;
 use crate::error::RadError;
-use crate::processor::{Processor, WriteOption};
+use crate::processor::Processor;
+use crate::models::WriteOption;
 use std::path::Path;
 use std::fs::OpenOptions;
 
@@ -16,8 +17,28 @@ impl Cli {
     }
 
     fn parse_options(args: &clap::ArgMatches) -> Result<(), RadError> {
+
+        let newline: String = 
+            if cfg!(target_os = "windows") && !args.is_present("newline"){
+                "\r\n".to_owned()
+            } else {
+                "\n".to_owned()
+            };
         // Processor
         let mut processor: Processor;
+        let mut error_option : Option<WriteOption> = Some(WriteOption::Stdout);
+        // Set error write option
+        if args.is_present("silent") {
+            error_option = None; 
+        } else if let Some(file) = args.value_of("err") {
+            let err_file = OpenOptions::new()
+                .create(true)
+                .write(true)
+                .truncate(true)
+                .open(file)
+                .unwrap();
+            error_option = Some(WriteOption::File(err_file));
+        }
         // Read from files
         if let Some(files) = args.values_of("FILE") {
             // Write to file
@@ -28,12 +49,13 @@ impl Cli {
                     .truncate(true)
                     .open(output_file)
                     .unwrap();
-                processor = Processor::new(WriteOption::File(out_file));
+                processor = Processor::new(WriteOption::File(out_file), error_option, newline);
             }
             // Write to standard output
-            else { processor = Processor::new(WriteOption::Stdout); }
+            else { processor = Processor::new(WriteOption::Stdout, error_option, newline); }
 
             for file in files {
+                processor.set_file(file);
                 processor.from_file(Path::new(file), false)?;
             }
         } 
@@ -46,8 +68,8 @@ impl Cli {
                     .truncate(true)
                     .open(output_file)
                     .unwrap();
-                processor = Processor::new(WriteOption::File(out_file));
-            } else { processor = Processor::new(WriteOption::Stdout); }
+                processor = Processor::new(WriteOption::File(out_file), error_option, newline);
+            } else { processor = Processor::new(WriteOption::Stdout, error_option, newline); }
 
             processor.from_stdin(false)?;
         }
@@ -60,11 +82,14 @@ impl Cli {
 
     fn args_builder() -> clap::ArgMatches {
         clap_app!(R4d =>
-            (version: "0.0.1")
+            (version: "0.1.5")
             (author: "Simon Creek <simoncreek@tutanota.com>")
             (about: "R4d is a modern macro processor made with rust")
             (@arg FILE: ... "Files to execute processing")
             (@arg out: -o --out +takes_value "File to print out macro")
+            (@arg err: -e --err +takes_value "File to save error")
+            (@arg silent: -s --silent "Supress warning")
+            (@arg newline: -n --newline "Use unix newline for formatting")
         ).get_matches()
     }
 }
