@@ -3,6 +3,11 @@ use crate::basic::BasicMacro;
 use crate::error::RadError;
 use crate::utils::Utils;
 
+pub enum WriteOption {
+    File(std::fs::File),
+    Stdout,
+}
+
 #[derive(Clone)]
 pub struct MacroRule{
     pub name: String,
@@ -26,13 +31,13 @@ impl MacroRule {
     }
 }
 
-pub struct MacroMap<'a> {
-    pub basic : BasicMacro<'a>,
+pub struct MacroMap {
+    pub basic : BasicMacro,
     pub custom : HashMap<String, MacroRule>,
     pub local : HashMap<String, String>,
 }
 
-impl<'a> MacroMap<'a> {
+impl MacroMap {
     pub fn new() -> Self {
         Self { 
             basic: BasicMacro::new(),
@@ -61,8 +66,35 @@ impl<'a> MacroMap<'a> {
         let mac = MacroRule::new(
             &Utils::trim(name)?, 
             &Utils::trim(args)?, 
-            &Utils::trim(body)?);
+            body);
         self.custom.insert(name.to_owned(), mac);
         Ok(())
     }
+
+    pub fn undefine(&mut self, name: &str) {
+        // Return true or false by the definition
+        if self.basic.contains(name) {
+            self.basic.undefine(name);
+        }
+        if self.custom.contains_key(name) {
+            self.custom.remove(name);
+        }
+    }
+
+    pub fn rename(&mut self, name: &str, target: &str) {
+        if self.basic.contains(name) {
+            self.basic.rename(name, target);
+        }
+        if self.custom.contains_key(name) {
+            let rule = self.custom.remove(name).unwrap();
+            self.custom.insert(target.to_owned(), rule);
+        }
+    }
+
+    pub fn append(&mut self, name: &str, target: &str) {
+        if self.custom.contains_key(name) {
+            let rule = self.custom.get_mut(name).unwrap();
+            rule.body.push_str(target);
+        }
+    }
 }
