@@ -1,24 +1,29 @@
 use std::io::{self, Write};
 use std::fs::File;
 use std::path::Path;
-use crate::error::RadError;
-use crate::models::MacroMap;
+use crate::error::{RadError, ErrorLogger};
+use crate::models::{MacroMap, WriteOption};
 use crate::utils::Utils;
 use crate::consts::*;
 use crate::lexor::*;
 
+#[derive(Debug)]
 pub struct MacroFragment {
     pub whole_string: String,
     pub name: String,
     pub args: String,
+    pub literal: bool,
+    pub pipe: bool,
 }
 
 impl MacroFragment {
     pub fn new() -> Self {
-        MacroFragment {  
+        MacroFragment {
             whole_string : String::new(),
             name : String::new(),
             args : String::new(),
+            literal : false,
+            pipe: false,
         }
     }
 
@@ -26,6 +31,8 @@ impl MacroFragment {
         self.whole_string.clear();
         self.name.clear();
         self.args.clear();
+        self.literal = false; 
+        self.pipe = false; 
     }
 
     pub fn is_empty(&self) -> bool {
@@ -39,26 +46,34 @@ pub enum ParseResult {
     NoPrint,
     EOI,
 }
-
-pub enum WriteOption {
-    File(std::fs::File),
-    Stdout,
-}
-
-pub struct Processor<'a> {
-    pub map: MacroMap<'a>,
+pub struct Processor{
+    pub map: MacroMap,
+    define_parse: DefineParser,
     write_option: WriteOption,
+    error_logger: ErrorLogger,
+    line_number: u64,
+    ch_number: u64,
+    pub pipe_value: String,
+    pub newline: String,
+    pub paused: bool,
 }
 // 1. Get string
 // 2. Parse until macro invocation detected
 // 3. Return remainder and macro fragments
 // 4. Continue parsing with fragments
 
-impl<'a> Processor<'a> {
-    pub fn new(write_option: WriteOption) -> Self {
+impl Processor {
+    pub fn new(write_option: WriteOption, error_write_option : Option<WriteOption>, newline: String) -> Self {
         Self {
             map : MacroMap::new(),
             write_option,
+            define_parse: DefineParser::new(),
+            error_logger: ErrorLogger::new(error_write_option),
+            line_number :0,
+            ch_number:0,
+            newline,
+            pipe_value: String::new(),
+            paused: false,
         }
     }
     pub fn get_map(&self) -> &MacroMap {
@@ -72,11 +87,14 @@ impl<'a> Processor<'a> {
         let mut content = String::new();
         let mut container = if get_result { Some(&mut content) } else { None };
         loop {
+            self.line_number = self.line_number + 1;
             let result = self.parse_line(&mut line_iter, &mut lexor ,&mut invoke)?;
+            // Clear local variable macros
+            self.map.clear_local();
             match result {
                 // This means either macro is not found at all
                 // or previous macro fragment failed with invalid syntax
-                ParseResult::Printable(mut remainder) => {
+                ParseResult::Printable(remainder) => {
                     self.write_to(&remainder, &mut container)?;
                     // Reset fragment
                     if &invoke.whole_string != "" {
@@ -104,7 +122,10 @@ impl<'a> Processor<'a> {
         let mut content = String::new();
         let mut container = if get_result { Some(&mut content) } else { None };
         loop {
+            self.line_number = self.line_number + 1;
             let result = self.parse_line(&mut line_iter, &mut lexor ,&mut invoke)?;
+            // Clear local variable macros
+            self.map.clear_local();
             match result {
                 // This means either macro is not found at all
                 // or previous macro fragment failed with invalid syntax
@@ -130,65 +151,8 @@ impl<'a> Processor<'a> {
     /// Parse line is called only by the main loop thus, caller name is special name of @MAIN@
     fn parse_line(&mut self, lines :&mut impl std::iter::Iterator<Item = std::io::Result<String>>, lexor : &mut Lexor ,frag : &mut MacroFragment) -> Result<ParseResult, RadError> {
         if let Some(line) = lines.next() {
-            // Rip off a result into a string
             let line = line?;
-            // Local values
-            let mut remainder = String::new();
-            let level = 0; // 0 is main level
-
-            // Reset lexor's escape_nl 
-            lexor.escape_nl = false;
-            for ch in line.chars() {
-                let lex_result = lexor.lex(ch)?;
-                // Either add character to remainder or fragments
-                match lex_result {
-                    LexResult::Ignore => frag.whole_string.push(ch),
-                    LexResult::AddToRemainder => {
-                        remainder.push(ch);
-                    }
-                    LexResult::AddToFrag(cursor) => {
-                        match cursor{
-                            Cursor::Name => frag.name.push(ch),
-                            Cursor::Arg => frag.args.push(ch),
-                            _ => unreachable!(),
-                        } 
-                        frag.whole_string.push(ch);
-                    }
-                    // End of fragment
-                    // 1. Evaluate macro 
-                    // 1.5 -> If define, parse rule applied again.
-                    // 2. And append to remainder
-                    // 3. Reset fragment
-                    LexResult::EndFrag => {
-                        frag.whole_string.push(ch);
-                        if frag.name == "define" {
-                            self.add_define(frag, &mut remainder)?;
-                            lexor.escape_nl = true;
-                        } else {
-                            // Invoke
-                            if let Some(content) = self.evaluate(level, &MAIN_CALLER.to_owned(), &frag.name, &frag.args)? {
-                                remainder.push_str(&content);
-                                frag.clear();
-
-                                // Clear local variable macros
-                                self.map.clear_local();
-                            } 
-                            // Failed to invoke
-                            // because macro doesn't exist
-                            else {
-                                remainder.push_str(&frag.whole_string);
-                                frag.clear()
-                            }
-                        }
-                    }
-                    // Remove fragment and set to remainder
-                    LexResult::ExitFrag => {
-                        frag.whole_string.push(ch);
-                        remainder.push_str(&frag.whole_string);
-                        frag.clear();
-                    }
-                }
-            } // End Character iteration
+            let remainder = self.parse(lexor, frag, &line, 0, MAIN_CALLER)?;
 
             // Non macro string is included
             if remainder.len() != 0 {
@@ -205,29 +169,79 @@ impl<'a> Processor<'a> {
             else {
                 Ok(ParseResult::NoPrint)
             }
-
         } else {
             Ok(ParseResult::EOI)
         }
     } // parse_line end
 
     /// Parse chunk is called by non-main process, thus needs caller
-    pub fn parse_chunk(&mut self, level: usize, caller: &String, chunk: &str) -> Result<String, RadError> {
+    pub fn parse_chunk(&mut self, level: usize, caller: &str, chunk: &str) -> Result<String, RadError> {
         let mut lexor = Lexor::new();
         let mut frag = MacroFragment::new();
+        let mut result = self.parse(&mut lexor, &mut frag, chunk, level, caller)?;
+        if !frag.is_empty() {
+            result.push_str(&frag.whole_string);
+        }
+        return Ok(result);
+    } // parse_chunk end
+
+    fn parse(&mut self,lexor: &mut Lexor, frag: &mut MacroFragment, line: &str, level: usize, caller: &str) -> Result<String, RadError> {
+        self.ch_number = 0;
+        // Local values
         let mut remainder = String::new();
-        for ch in chunk.chars() {
+
+        // Reset lexor's escape_nl 
+        lexor.escape_nl = false;
+        for ch in line.chars() {
+            self.ch_number = self.ch_number + 1;
             let lex_result = lexor.lex(ch)?;
             // Either add character to remainder or fragments
             match lex_result {
                 LexResult::Ignore => frag.whole_string.push(ch),
+                LexResult::StartFrag => {
+                    frag.whole_string.push(ch);
+
+                    // If paused and not pause, then reset lexor context
+                    if self.paused && frag.name != "pause" {
+                        lexor.reset();
+                        remainder.push_str(&frag.whole_string);
+                        frag.clear();
+                    }
+                },
+                LexResult::EmptyName => {
+                    frag.whole_string.push(ch);
+                    self.error_logger
+                        .set_number(
+                            self.line_number, 
+                            self.ch_number
+                        );
+                    // If paused, then reset lexor context
+                    if self.paused {
+                        lexor.reset();
+                        remainder.push_str(&frag.whole_string);
+                        frag.clear();
+                    }
+                }
                 LexResult::AddToRemainder => {
                     remainder.push(ch);
                 }
                 LexResult::AddToFrag(cursor) => {
                     match cursor{
-                        Cursor::Name => frag.name.push(ch),
-                        Cursor::Arg => frag.args.push(ch),
+                        Cursor::Name => {
+                            if frag.name.len() == 0 {
+                                self.error_logger
+                                    .set_number(
+                                        self.line_number, 
+                                        self.ch_number
+                                    );
+                            }
+                            if ch == '\\' { frag.literal = true; }
+                            else if ch == '|' { frag.pipe = true; }
+                            else { frag.name.push(ch); }
+                        },
+                        Cursor::Arg => {
+                            frag.args.push(ch)
+                        },
                         _ => unreachable!(),
                     } 
                     frag.whole_string.push(ch);
@@ -239,15 +253,38 @@ impl<'a> Processor<'a> {
                 // 3. Reset fragment
                 LexResult::EndFrag => {
                     frag.whole_string.push(ch);
+                    // Literal rule
+                    if frag.literal {
+                        frag.args = Utils::escape_all(&frag.args)?;
+                    }
+                    // define
                     if frag.name == "define" {
-                        self.add_define(&mut frag, &mut remainder)?;
+                        self.add_define(frag, &mut remainder)?;
                         lexor.escape_nl = true;
-                    } else {
-                        // Invoke
-                        if let Some(content) = self.evaluate(level + 1, caller, &frag.name, &frag.args)? {
-                            remainder.push_str(&content);
-                            frag.clear();
+                        frag.clear()
+                    } 
+                    // Invoke macro
+                    else {
+                        if let Some(content) = self.evaluate(level, caller, &frag.name, &frag.args)? {
+                            if frag.pipe {
+                                self.pipe_value = content;
+                                lexor.escape_nl = true;
+                            }
+                            // If content is none
+                            // Ignore new line after macro evaluation until any character
+                            else if content.len() == 0 {
+                                lexor.escape_nl = true;
+                            } else {
+                                remainder.push_str(&content);
+                            }
+                        } 
+                        // Failed to invoke
+                        // because macro doesn't exist
+                        else {
+                            remainder.push_str(&frag.whole_string);
                         }
+                        // Clear fragment regardless of success
+                        frag.clear()
                     }
                 }
                 // Remove fragment and set to remainder
@@ -257,16 +294,17 @@ impl<'a> Processor<'a> {
                     frag.clear();
                 }
             }
-        } // End character iteration
-        return Ok(remainder)
-    } // parse_chunk end
+        } // End Character iteration
+        Ok(remainder)
+    }
 
     fn add_define(&mut self, frag: &mut MacroFragment, remainder: &mut String) -> Result<(), RadError> {
-        // Failed to register macro
-        if let Some((name,args,body)) = Self::parse_define(&frag.args) {
+        if let Some((name,args,body)) = self.define_parse.parse_define(&frag.args) {
             self.map.register(&name, &args, &body)?;
         } else {
-            eprintln!("Failed to register a macro : {}", frag.name);
+            self.log_error(&format!(
+                    "Failed to register a macro : \"{}\"", frag.args.split(',').collect::<Vec<&str>>()[0]
+            ))?;
             remainder.push_str(&frag.whole_string);
         }
         // Clear fragment regardless of success
@@ -275,96 +313,14 @@ impl<'a> Processor<'a> {
         Ok(())
     }
 
-    // Static function
-    // NOTE This method expects valid form of macro invocation
-    fn parse_define(text: &str) -> Option<(String, String, String)> {
-        let mut arg_cursor = "name";
-        let mut name = String::new();
-        let mut args = String::new();
-        let mut body = String::new();
-
-        let mut container = String::new();
-
-        for ch in text.chars() {
-            if ch == ',' {
-                match arg_cursor {
-                    "name" => {
-                        name.push_str(&container);
-                        container.clear();
-                        arg_cursor = "args"; 
-                        continue;
-                    },
-                    "args" => {
-                        args.push_str(&container);
-                        container.clear();
-                        arg_cursor = "body"; 
-                        continue;
-                    },
-                    "body" => {
-                        body.push_str(&container);
-                        container.clear();
-                        return Some((name, args, body));
-                    }
-                    _ => unreachable!()
-                }
-            }
-            if ch == ' ' || ch == '\n' || ch == '\r' || ch == '\t' {
-                // This means pattern like this
-                // $define( name ) -> name is registered
-                // $define( na me ) -> only "na" is registered
-                if arg_cursor == "name" && name.len() != 0 {
-                    name.push_str(&container);
-                    container.clear();
-                    arg_cursor = "args";
-                    continue;
-                } 
-                // This means pattern like this
-                // $define(name, arg1 ,)
-                //                   |
-                //                  -This part makes argument empty
-                //                  and starts argument evaluation as new state
-                else if arg_cursor == "args" && container.len() != 0 {
-                    args.push_str(&container);
-                    args.push(' ');
-                    container.clear();
-                    continue;
-                }
-            } else {
-                // Body can be any text but, 
-                // name and arg should follow rules
-                if arg_cursor != "body" {
-                    if container.len() == 0 { // Start of string
-                        // Not alphabetic 
-                        // $define( 1name ) -> Not valid
-                        if !ch.is_alphabetic() {
-                            return None;
-                        }
-                    } else { // middle of string
-                        // Not alphanumeric and not underscore
-                        // $define( na*1me ) -> Not valid
-                        // $define( na_1me ) -> Valid
-                        if !ch.is_alphanumeric() && ch != '_' {
-                            return None;
-                        }
-                    }
-                }
-            }
-
-            container.push(ch);
-        }
-
-        // End of body
-        body.push_str(&container);
-
-        Some((name, args, body))
-    }
-
     // Evaluate can be nested deeply
-    fn evaluate(&mut self,level: usize, caller: &String, name: &String, args: &String) -> Result<Option<String>, RadError> {
-
+    fn evaluate(&mut self,level: usize, caller: &str, name: &str, args: &String) -> Result<Option<String>, RadError> {
+        let level = level + 1;
         // This parses and processes arguments
         // and macro should be evaluated after
-        let args = self.parse_chunk(level, caller, args)?; 
+        // TODO 
+        // Make caller to name
+        let args = self.parse_chunk(level, name, args)?; 
 
         // Find Local macro first
         if let Some(local) = self.map.local.get(&Utils::local_name(level, &caller, &name)) {
@@ -387,11 +343,12 @@ impl<'a> Processor<'a> {
         } 
         // No macros found to evaluate
         else { 
+            self.log_error(&format!("Failed to invoke a macro : \"{}\"", name))?;
             return Ok(None);
         }
     }
 
-    fn invoke_rule(&mut self,level: usize ,name: &String, arg_values: &String) -> Result<Option<String>, RadError> {
+    fn invoke_rule(&mut self,level: usize ,name: &str, arg_values: &str) -> Result<Option<String>, RadError> {
         // Get rule
         // Invoke is called only when key exists, thus unwrap is safe
         let rule = self.map.custom.get(name).unwrap().clone();
@@ -401,8 +358,7 @@ impl<'a> Processor<'a> {
 
         // Necessary arg count is bigger than given arguments
         if arg_types.len() > arg_values.len() {
-            eprintln!("Arg types : {:?}\nArg values : {:?}", arg_types, arg_values);
-            eprintln!("{}'s arguments are not sufficient", name);
+            self.log_error(&format!("{}'s arguments are not sufficient. Given {}, but needs {}", name, arg_values.len(), arg_types.len()))?;
             return Ok(None);
         }
 
@@ -431,4 +387,154 @@ impl<'a> Processor<'a> {
 
         Ok(())
     }
+
+    fn log_error(&mut self, log : &str) -> Result<(), RadError> {
+        self.error_logger.elog(log)?;
+        Ok(())
+    }
+
+    pub fn set_file(&mut self, file: &str) {
+        self.error_logger.set_file(file);
+        self.line_number = 0;
+        self.ch_number = 0;
+    }
+}
+
+pub struct DefineParser{
+    arg_cursor :DefineCursor,
+    name: String,
+    args: String,
+    body: String,
+    dquote: bool,
+    container: String,
+}
+
+impl DefineParser {
+    pub fn new() -> Self {
+        Self {
+            arg_cursor : DefineCursor::Name,
+            name : String::new(),
+            args : String::new(),
+            body : String::new(),
+            dquote : false,
+            container : String::new(),
+        }
+    }
+
+    fn clear(&mut self) {
+        self.arg_cursor = DefineCursor::Name;
+        self.name.clear();
+        self.args.clear();
+        self.body.clear();
+        self.dquote = false;
+        self.container.clear();
+    }
+
+    // Static function
+    // NOTE This method expects valid form of macro invocation
+    // Given value should be without outer prentheses
+    // e.g. ) name,a1 a2,body text
+    pub fn parse_define(&mut self, text: &str) -> Option<(String, String, String)> {
+        self.clear();
+        for ch in text.chars() {
+            match self.arg_cursor {
+                DefineCursor::Name => {
+                    // $define(variable=something)
+                    // Don't set argument but directly bind variable to body
+                    if ch == '=' {
+                        self.name.push_str(&self.container);
+                        self.container.clear();
+                        self.arg_cursor = DefineCursor::Body;
+                        continue;
+                    } 
+                    // This means pattern like this
+                    // $define( name ) -> name is registered
+                    // $define( na me ) -> only "na" is registered
+                    else if Utils::is_blank_char(ch) && self.name.len() != 0 {
+                        self.name.push_str(&self.container);
+                        self.container.clear();
+                        self.arg_cursor = DefineCursor::Args;
+                    } 
+                    // Comma go to args
+                    // Dquote is invalid character so no need to escape
+                    else if ch == ',' {
+                        self.name.push_str(&self.container);
+                        self.container.clear();
+                        self.arg_cursor = DefineCursor::Args;
+                        continue;
+                    } else {
+                        // If not valid name return None
+                        if !self.is_valid_name(ch) { return None; }
+                    }
+                }
+                DefineCursor::Args => {
+                    // Blank space separates arguments 
+                    if Utils::is_blank_char(ch) && self.name.len() != 0 {
+                        self.args.push_str(&self.container);
+                        self.args.push(' ');
+                        self.container.clear();
+                        continue;
+                    } 
+                    // Go to body
+                    // Dquote is invalid character so no need to escape
+                    else if ch == ',' {
+                        self.args.push_str(&self.container);
+                        self.container.clear();
+                        self.arg_cursor = DefineCursor::Body; 
+                        continue;
+                    } 
+                    // Others
+                    else {
+                        // If not valid name return
+                        if !self.is_valid_name(ch) { return None; }
+                    }
+                }
+                // Add everything
+                DefineCursor::Body => ()
+            } 
+            self.container.push(ch);
+        }
+
+        // End of body
+        self.body.push_str(&self.container);
+
+        Some((self.name.clone(), self.args.clone(), self.body.clone()))
+    }
+
+    fn is_valid_name(&mut self, ch : char) -> bool {
+        if self.container.len() == 0 { // Start of string
+            // Not alphabetic 
+            // $define( 1name ) -> Not valid
+            if !ch.is_alphabetic() {
+                return false;
+            }
+        } else { // middle of string
+            // Not alphanumeric and not underscore
+            // $define( na*1me ) -> Not valid
+            // $define( na_1me ) -> Valid
+            if !ch.is_alphanumeric() && ch != '_' {
+                return false;
+            }
+        }
+        true
+    }
+    
+    // Reserved for later refactoring
+    //fn branch_name() {
+
+    //}
+
+    //fn branch_args() {
+
+    //}
+
+    //fn branch_body() {
+
+    //}
+}
+
+enum DefineCursor {
+    Name,
+    Args,
+    Body,
 }
